@@ -11,14 +11,14 @@ Interchange6::Cart - Cart class for Interchange6 Shop Machine
 use strict;
 use Carp;
 use DateTime;
-use Interchange6::Cart::Cost;
 use Interchange6::Cart::Product;
 use Scalar::Util 'blessed';
 use Try::Tiny;
 use Moo;
 use MooseX::CoverableModifiers;
 use MooX::HandlesVia;
-use Interchange6::Types;
+use Types::Standard qw/ArrayRef InstanceOf Str/;
+use Types::Common::String qw/NonEmptyStr/;
 
 with 'Interchange6::Role::Costs';
 
@@ -46,6 +46,8 @@ Generic cart class for L<Interchange6>.
 
 =head1 ATTRIBUTES
 
+See also L<Interchange6::Role::Costs/ATTRIBUTES>.
+
 =head2 id
 
 Cart id can be used for subclasses, e.g. primary key value for carts in the database.
@@ -77,21 +79,27 @@ The cart name. Default is 'main'.
 =cut
 
 has name => (
-    is       => 'ro',
-    isa      => AllOf [ Defined, NotEmpty, VarChar [255] ],
-    default  => 'main',
-    writer   => 'rename',
+    is      => 'ro',
+    isa     => NonEmptyStr,
+    default => 'main',
+    writer  => 'rename',
 );
 
 =head2 products
 
-Called without args returns a hash reference of L<Interchange6::Cart::Product>. Should not normally be called with args but rather via the various L</PRODUCT METHODS> detailed below.
+Called without args returns a hash reference of L<Interchange6::Cart::Product>.
+
+Anything passed in as a value on object instantiation is ignored. To load
+products into a cart the preferred methods are L</add> and L</seed> which
+make sure appropriate arguements are passed.
 
 =cut
 
 has products => (
+    # rwp allows us to clear out products in seed via _set_products
+    # without disturbing what subclasses might expect of clear
     is  => 'rwp',
-    isa => ArrayRef [ InstanceOf ['Interchange::Cart::Product'] ],
+    isa => ArrayRef [ InstanceOf ['Interchange6::Cart::Product'] ],
     default     => sub { [] },
     handles_via => 'Array',
     handles     => {
@@ -101,9 +109,9 @@ has products => (
         product_get    => 'get',
         product_index  => 'first_index',
         products_array => 'elements',
-        _delete        => 'delete',
-        _product_push  => 'push',
-        _product_set   => 'set',
+        product_delete => 'delete',
+        product_push   => 'push',
+        product_set    => 'set',
     },
     init_arg => undef,
 );
@@ -122,6 +130,7 @@ The session ID for the cart.
 
 has sessions_id => (
     is      => 'ro',
+    isa     => Str,
     clearer => 1,
     writer  => 'set_sessions_id',
 );
@@ -148,25 +157,17 @@ sub _build_subtotal {
     return sprintf( "%.2f", $subtotal );
 }
 
-=head2 total
+after 'add', 'clear', 'product_push', 'product_set', 'product_delete', 'remove',
+  'seed', 'update' => sub {
 
-Returns current cart total including costs.
-
-=cut
-
-has total => (
-    is        => 'lazy',
-    clearer   => 1,
-    predicate => 1,
-);
-
-sub _build_total {
     my $self = shift;
+    $self->clear_subtotal;
+    $self->clear_weight;
+  };
 
-    my $subtotal = $self->subtotal;
-
-    return sprintf( "%.2f", $subtotal + $self->_calculate($subtotal) );
-}
+after 'clear_subtotal' => sub {
+    shift->clear_total;
+};
 
 =head2 users_id
 
@@ -206,31 +207,17 @@ sub _build_weight {
     map { $weight += $_->weight * $_->quantity }
       grep { defined $_->weight } $self->products_array;
 
-    return $weight;
+    return $weight ? $weight : 0;
 }
 
 =head1 METHODS
 
-See L<Interchange6::Role::Costs> for details of cost attributes and methods.
+See also L<Interchange6::Role::Costs/METHODS>.
 
 =head2 clear
 
 Removes all products from the cart.
 
-=cut
-
-around clear => sub {
-    my ( $orig, $self ) = @_;
-
-    # fire off the clear
-    $orig->( $self, @_ );
-    $self->clear_subtotal;
-    $self->clear_total;
-    $self->clear_weight;
-
-    return;
-};
-
 =head2 count
 
 Returns the number of different products in the shopping cart. If you have 5 apples and 6 pears it will return 2 (2 different products).
@@ -239,9 +226,13 @@ Returns the number of different products in the shopping cart. If you have 5 app
 
 Return boolean 1 or 0 depending on whether the cart is empty or not.
 
+=head2 product_delete($index)
+
+Deletes the product at the specified index.
+
 =head2 product_get($index)
 
-Returns the product at the specified index;
+Returns the product at the specified index.
 
 =head2 product_index( sub {...})
 
@@ -251,6 +242,16 @@ This method requires a single argument.
 
   my $index = $cart->product_index( sub { $_->sku eq 'ABC' } );
 
+=head2 product_push($product)
+
+Like Perl's normal C<push> this adds the supplied L<Interchange::Cart::Product>
+to L</products>.
+
+=head2 product_set($index, $product)
+
+Sets the product at the specified index in L</products> to the supplied
+L<Interchange::Cart::Product>.
+
 =head2 products_array
 
 Returns an array of Interchange::Cart::Product(s)
@@ -272,20 +273,26 @@ sub add {
     my $product = $_[0];
     my ( $index, $oldproduct, $update );
 
-    if ( blessed($product) ) {
+    if ( !defined $product ) {
+        die "undefined argument passed to add";
+    }
+    elsif ( blessed($product) ) {
         die "product argument is not an Interchange6::Cart::Product"
           unless ( $product->isa('Interchange6::Cart::Product') );
     }
     else {
 
-        # we got a hash(ref) rather than an Product
-
         my %args;
 
-        if ( is_HashRef($product) ) {
+        if ( @_ % 2 ) {
+            if ( ref($product) eq 'HASH' ) {
 
-            # copy args
-            %args = %{$product};
+                # copy args
+                %args = %{$product};
+            }
+            else {
+                die "argument to add should be hash or hashref";
+            }
         }
         else {
 
@@ -293,12 +300,6 @@ sub add {
         }
 
         $product = 'Interchange6::Cart::Product'->new(%args);
-
-        unless ( blessed($product)
-            && $product->isa('Interchange6::Cart::Product') )
-        {
-            die "failed to create cart product.";
-        }
     }
 
    # cart may already contain an product with the same sku
@@ -314,7 +315,7 @@ sub add {
 
         $product->set_quantity( $oldproduct->quantity + $product->quantity );
 
-        $self->_product_set( $index, $product );
+        $self->product_set( $index, $product );
 
         $update = 1;
     }
@@ -322,14 +323,10 @@ sub add {
 
         # a new product for this cart
 
-        $product->set_cart( $self );
-        $self->_product_push($product);
+        $product->set_cart($self);
+        $self->product_push($product);
     }
 
-    $self->clear_subtotal;
-    $self->clear_total;
-    $self->clear_weight;
-
     return $product;
 }
 
@@ -395,15 +392,16 @@ Remove product from the cart. Takes SKU of product to identify the product.
 sub remove {
     my ( $self, $arg ) = @_;
 
+    die "no argument passed to remove" unless defined $arg;
+
     my $index = $self->product_index( sub { $_->sku eq $arg } );
 
+    die "sku $arg not found in cart" unless $index >= 0;
+
     # remove product from our array
-    my $ret = $self->_delete($index);
+    my $ret = $self->product_delete($index);
     die "remove sku $arg failed" unless defined $ret;
 
-    $self->clear_subtotal;
-    $self->clear_total;
-    $self->clear_weight;
     return $ret;
 }
 
@@ -419,26 +417,30 @@ B<NOTE:> use with caution since any existing products in the cart will be lost.
       { sku => 'DBF2020', price => 200, quantity = 5 },
   ]);
 
+If any product fails to be added (for example bad product args) then an
+exception is thrown and no products will be added to cart.
+
+On success returns L</products>.
+
 =cut
 
 sub seed {
     my ( $self, $product_ref ) = @_;
-    my ( $args, $product );
 
-  PRODUCT: for $args ( @{ $product_ref || [] } ) {
+    die "argument to seed must be an array reference"
+      unless ref($product_ref) eq 'ARRAY';
 
-        $product = Interchange6::Cart::Product->new($args);
-        unless ( blessed($product)
-            && $product->isa('Interchange6::Cart::Product') )
-        {
-            die "failed to create product.";
-        }
+    # we can't use ->clear since subclasses might wrap that to do
+    # interesting things like remove products from cart in database
+    $self->_set_products([]);
 
-        $self->_product_push($product);
+    my @products;
+    for my $args ( @{$product_ref} ) {
+        push @products, Interchange6::Cart::Product->new($args);
     }
-    $self->clear_subtotal;
-    $self->clear_total;
-    $self->clear_weight;
+
+    # if we got here then all products are good so put them in the cart
+    $self->product_push(@products);
 
     return $self->products;
 }
@@ -454,9 +456,9 @@ Parameters are pairs of SKUs and quantities, e.g.
 
 A quantity of zero is equivalent to removing this product.
 
-Returns updated products that are still in the cart. Products removed
-via quantity 0 or products for which quantity has not changed will not
-be returned.
+Returns an array of updated products that are still in the cart.
+Products removed via quantity 0 or products for which quantity has not
+changed will not be returned.
 
 =cut
 
@@ -468,6 +470,11 @@ sub update {
         $sku = shift @args;
         $qty = shift @args;
 
+        die "sku not defined in arg to update" unless defined $sku;
+
+        die "quantity not supplied as arg to update for sku $sku"
+          unless defined $qty;
+
         unless ( $product = $self->find($sku) ) {
             die "Product for $sku not found in cart.";
         }
@@ -481,27 +488,11 @@ sub update {
         next ARGS if $qty == $product->quantity;
 
         $product->set_quantity($qty);
-        $self->clear_subtotal;
-        $self->clear_total;
-        $self->clear_weight;
         push @products, $product;
     }
-    return wantarray ? @products : \@products;
+    return @products;
 }
 
-# after cost changes we need to clear the total
-
-after apply_cost => sub {
-    my $self = shift;
-    $self->clear_subtotal;
-    $self->clear_total;
-};
-after clear_costs => sub {
-    my $self = shift;
-    $self->clear_subtotal;
-    $self->clear_total;
-};
-
 =head1 AUTHORS
 
  Stefan Hornburg (Racke), <racke@linuxia.de>
