@@ -4,8 +4,11 @@ package Interchange6::Cart::Product;
 
 use strict;
 use Moo;
+use MooseX::CoverableModifiers;
 use MooX::HandlesVia;
-use Interchange6::Types;
+use Types::Standard qw/Defined HashRef InstanceOf Int Num Str Undef/;
+use Types::Common::Numeric qw/PositiveInt PositiveOrZeroNum/;
+use Types::Common::String qw/NonEmptyStr/;
 with 'Interchange6::Role::Costs';
 
 use namespace::clean;
@@ -22,6 +25,8 @@ See L<Interchange6::Role::Costs> for details of cost attributes and methods.
 
 =head1 ATTRIBUTES
 
+See also L<Interchange6::Role::Costs/ATTRIBUTES>.
+
 Each cart product has the following attributes:
 
 =head2 id
@@ -49,6 +54,7 @@ A reference to the Cart object that this Cart::Product belongs to.
 
 has cart => (
     is      => 'ro',
+    isa     =>  Undef | InstanceOf ['Interchange6::Cart'],
     default => undef,
     writer  => 'set_cart',
 );
@@ -61,7 +67,7 @@ Product name is required.
 
 has name => (
     is       => 'ro',
-    isa      => AllOf [ Defined, NotEmpty, VarChar [255] ],
+    isa      => Defined & NonEmptyStr,
     required => 1,
 );
 
@@ -81,7 +87,7 @@ Price is required, because you want to maintain the price that was valid at the
 
 has price => (
     is        => 'ro',
-    isa       => AnyOf [ PositiveNum, Zero ],
+    isa       => PositiveOrZeroNum,
     required  => 1,
     writer    => 'set_price',
 );
@@ -100,7 +106,7 @@ Selling price is the price after group pricing, tier pricing or promotional disc
 
 has selling_price => (
     is        => 'lazy',
-    isa       => AnyOf [ PositiveNum, Zero ],
+    isa       => PositiveOrZeroNum,
     writer    => 'set_selling_price',
 );
 
@@ -115,10 +121,14 @@ This is the integer discount percentage calculated from the difference
 between L</price> and L</selling_price>. This attribute should not normally
 be set since as it is a calculated value.
 
+L</discount_percent> is cleared if either L</set_price> or
+L<set_selling_price> methods are called.
+
 =cut
 
 has discount_percent => (
-    is => 'lazy',
+    is      => 'lazy',
+    clearer => 1
 );
 
 sub _build_discount_percent {
@@ -127,6 +137,10 @@ sub _build_discount_percent {
     return int( ( $self->price - $self->selling_price ) / $self->price * 100 );
 }
 
+after 'set_price', 'set_selling_price' => sub {
+    shift->clear_discount_percent;
+};
+
 =head2 quantity
 
 Product quantity is optional and has to be a natural number greater
@@ -136,7 +150,7 @@ than zero. Default for quantity is 1.
 
 has quantity => (
     is      => 'ro',
-    isa     => AllOf [ PositiveNum, Int ],
+    isa     => PositiveInt,
     default => 1,
     writer  => 'set_quantity',
 );
@@ -155,7 +169,7 @@ Unique product identifier is required.
 
 has sku => (
     is       => 'ro',
-    isa      => AllOf [ Defined, NotEmpty, VarChar [32] ],
+    isa      => NonEmptyStr,
     required => 1,
 );
 
@@ -189,25 +203,6 @@ sub _build_subtotal {
     return sprintf( "%.2f", $self->selling_price * $self->quantity);
 }
 
-=head2 total
-
-Total calculated as L</subtotal> plus all L<Interchange6::Role:Costs/costs>.
-
-=cut
-
-has total => (
-    is        => 'lazy',
-    isa       => Num,
-    clearer   => 1,
-    predicate => 1,
-);
-
-sub _build_total {
-    my $self = shift;
-    my $subtotal = $self->subtotal;
-    return sprintf( "%.2f", $subtotal + $self->_calculate($subtotal) );
-}
-
 =head2 uri
 
 Product uri
@@ -216,7 +211,7 @@ Product uri
 
 has uri => (
     is  => 'ro',
-    isa => VarChar [255],
+    isa => Str,
 );
 
 =head2 weight
@@ -227,7 +222,7 @@ Weight of quantity 1 of this product.
 
 has weight => (
     is     => 'ro',
-    isa    => AnyOf [ Undef, Num ],
+    isa    => Num,
     writer => 'set_weight',
 );
 
@@ -263,6 +258,8 @@ has extra => (
 
 =head1 METHODS
 
+See also L<Interchange6::Role::Costs/METHODS>.
+
 =head2 L</extra> methods
 
 =over
@@ -297,7 +294,6 @@ See L<Data::Perl::Role::Collection::Hash/defined>
 
 =back
 
-
 =head2 L</subtotal> methods
 
 =over
@@ -312,21 +308,7 @@ predicate on L</subtotal>.
 
 =back
 
-
-=head2 L</total> methods
-
-=over
-
-=item * clear_total
-
-Clears L</total>.
-
-=item * has_total
-
-predicate on L</total>.
-
-
-=item * is_variant
+=head2 is_variant
 
 Returns 1 if L</canonical_sku> is defined else 0.
 
@@ -336,7 +318,7 @@ sub is_variant {
     return defined shift->canonical_sku ? 1 : 0;
 }
 
-=item * is_canonical
+=head2 is_canonical
 
 Returns 0 if L</canonical_sku> is defined else 1.
 
@@ -346,27 +328,20 @@ sub is_canonical {
     return defined shift->canonical_sku ? 0 : 1;
 }
 
-=back
+# after cost changes we need to clear the cart subtotal/total
+# our own total is handled by the Costs role
 
-=cut
-
-# after cost changes we need to clear the total as well as cart subtotal/total
-
-after apply_cost => sub {
+after 'clear_costs', 'cost_set', 'cost_push', 'set_quantity' => sub {
     my $self = shift;
-    $self->clear_total;
     if ( $self->cart ) {
         $self->cart->clear_subtotal;
         $self->cart->clear_total;
     }
 };
-
-after clear_costs => sub {
+after 'set_quantity', 'set_weight' => sub {
     my $self = shift;
-    $self->clear_total;
     if ( $self->cart ) {
-        $self->cart->clear_subtotal;
-        $self->cart->clear_total;
+        $self->cart->clear_weight;
     }
 };
 
